/*******************************************************************************
 * Copyright 2019 Graz University of Technology
 * MOA ZS has been developed in a cooperation between EGIZ
 * and Graz University of Technology.
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
package at.gv.egiz.moazs;

import at.gv.zustellung.msg.xsd.PreAdviceNoteSentType;
import org.springframework.lang.Nullable;

/**
 * @author Christof Rabensteiner
 *
 */
public class MoaZSException extends RuntimeException {

    public static final String ERROR_MZS_MIMETYPE_MISSMATCH = "8001";
    public static final String ERROR_MZS_NO_TNVZ_PERSON_QUERY_RESULTS = "8002";
    public static final String ERROR_MZS_BINARY_RESPONSE_MISSING = "8003";
    public static final String ERROR_MOASP_SIGNATURE_INVALID = "7001";

    @Nullable private final String code;
    @Nullable private final PreAdviceNoteSentType preAdviceNoteSent;

    private MoaZSException(String message, Throwable cause, @Nullable String code, @Nullable PreAdviceNoteSentType preAdviceNoteSent) {
        super(message, cause);
        this.code = code;
        this.preAdviceNoteSent = preAdviceNoteSent;
    }

    @Nullable public String getCode() {
        return code;
    }

    @Nullable public PreAdviceNoteSentType getPreAdviceNoteSent() {
        return preAdviceNoteSent;
    }

    public static MoaZSException moaZSException(String message, Throwable cause) {
        return new MoaZSException(message, cause, null, null);
    }

    public static MoaZSException moaZSException(String message) {
        return new MoaZSException(message, null, null, null);
    }

    public static MoaZSException moaZSException(String message, String code) {
        return new MoaZSException(message, null, code, null);
    }

    public static MoaZSException moaZSException(String message, String code, Throwable cause) {
        return new MoaZSException(message, cause, code, null);
    }

    public static MoaZSException moaZSException(String message, String code, PreAdviceNoteSentType preAdviceNoteSent) {
        return new MoaZSException(message, null, code, preAdviceNoteSent);
    }

}
